;; Generated package description from myers.el  -*- no-byte-compile: t -*-
(define-package "myers" "0.1" "Random-access singly-linked lists" '((emacs "25")) :commit "cc8d39d05c4a48545336510df7ac9ab186611ea2" :url "https://elpa.gnu.org/packages/myers.html" :authors '(("Stefan Monnier" . "monnier@iro.umontreal.ca")) :maintainer '("Stefan Monnier" . "monnier@iro.umontreal.ca") :keywords '("list" "containers"))
